Package: MDSvis
Title: Plots of Multi Dimensional Scaling (MDS) results
Version: 1.1.0
Authors@R: 
    c(person(given = "Andrea",
             family = "Vicini",
             role = c("aut"),
             email = "andrea.vicini@uclouvain.be",
             comment = c(ORCID = "0000-0001-9438-6909")),
      person(given = "Philippe",
             family = "Hauchamps",
             role = c("aut", "cre"),
             email = "philippe.hauchamps@uclouvain.be",
             comment = c(ORCID = "0000-0003-2865-1852")),
      person(given = "Shabnam",
             family = "Zaman",
             role = "ctb",
             email = "shabnam.zaman@uclouvain.be",
             comment = c(ORCID = "0000-0001-5399-0130")),
      person(given = "Laurent",
             family = "Gatto",
             role = "aut",
             email = "laurent.gatto@uclouvain.be",
             comment = c(ORCID = "0000-0002-1520-2268")))
Description: This package implements visulization of Multi Dimensional Scaling
             (MDS) results.
License: GPL-3
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
BugReports: https://github.com/UCLouvain-CBIO/MDSvis/issues
URL: https://uclouvain-cbio.github.io/MDSvis
biocViews: FlowCytometry, QualityControl, DimensionReduction,
        MultidimensionalScaling, Software, Visualization
Depends: R (>= 4.6)
Imports: CytoMDS (>= 1.3.5), rlang, ggplot2, plotly, shiny, shinyjs,
        methods
Suggests: knitr, rmarkdown, BiocStyle, HDCytoData, flowCore, testthat
        (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/MDSvis
git_branch: devel
git_last_commit: cd89dfa
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:37:31 UTC; biocbuild
Author: Andrea Vicini [aut] (ORCID: <https://orcid.org/0000-0001-9438-6909>),
  Philippe Hauchamps [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2865-1852>),
  Shabnam Zaman [ctb] (ORCID: <https://orcid.org/0000-0001-5399-0130>),
  Laurent Gatto [aut] (ORCID: <https://orcid.org/0000-0002-1520-2268>)
Maintainer: Philippe Hauchamps <philippe.hauchamps@uclouvain.be>

# Generated by roxygen2: do not edit by hand

export(mdsvis_app)
import(shiny)
importFrom(CytoMDS,ggplotSampleMDS)
importFrom(CytoMDS,nDim)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,labs)
importFrom(methods,is)
importFrom(plotly,ggplotly)
importFrom(plotly,plotlyOutput)
importFrom(plotly,renderPlotly)
importFrom(rlang,.data)
importFrom(shinyjs,disable)
importFrom(shinyjs,enable)
importFrom(shinyjs,useShinyjs)

# MDSvis

The `MDSvis` package implements visualisation of Multi Dimensional Scaling (MDS)
objects representing a low dimensional projection of cytometry samples.
Such objects can be obtained using the functionalities implemented in the
`CytoMDS` package. The visualisation is realised via a Shiny app that allows 
the user to interactively customise the plots depending on a series of input
parameters.

Below a screenshot of the app 'View' tab. The controls on the side allow to
choose the projection axes, colour, label, facet or shape the points 
according to phenodata variables, add biplot, 
show a `plotly` plot for interactive plot exploration 
or flip axes.

![](vignettes/images/ViewPlotly.png)

### Installation

To install the package run the following lines:

```r
if (!require("devtools", quietly = TRUE))
    install.packages("devtools")

devtools::install_github(repo = 'https://github.com/UCLouvain-CBIO/MDSvis')
```

Note that `CytoMDS` version 1.3.5 or above needs to be installed before.

### License

The `MDSvis` code is provided under [GPL license version 3.0 or 
higher](https://opensource.org/licenses/GPL-3.0). The documentation, 
including the manual pages and the vignettes, are distributed under a [CC BY-SA 
4.0 license](https://creativecommons.org/licenses/by-sa/4.0/).

### Citation

If you use `MDSvis` in your research, please use the following citation:

>Hauchamps, Philippe, Simon Delandre, Stéphane T. Temmerman, 
> Dan Lin, and Laurent Gatto. 2025. 
> “Visual Quality Control with CytoMDS, a Bioconductor Package 
> for Low Dimensional Representation of Cytometry Sample Distances.” 
> Cytometry. Part A: 
> The Journal of the International Society for Analytical Cytology 107 (3): 177–86.

# MDSvis - Copyright (C) <2025-2026>
# <Université catholique de Louvain (UCLouvain), Belgique>
#
#   Description and complete License: see LICENSE file.
#
# This program (MDSvis) is free software:
#   you can redistribute it and/or modify it under the terms of the GNU General
# Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details (<http://www.gnu.org/licenses/>).
#


#' @export
#' @title Launch shiny app for MDS projection visualization
#' @param preLoadDemoDataset if TRUE, pre-load the *Krieg_Anti_PD_1* dataset
#' @import shiny
#' @importFrom shinyjs useShinyjs enable disable
#' @importFrom plotly renderPlotly plotlyOutput ggplotly
#' @importFrom CytoMDS nDim ggplotSampleMDS
#' @importFrom ggplot2 facet_wrap ggsave labs
#' @importFrom rlang .data
#' @return no return value
#' @examples
#' if (interactive()) {
#'   mdsvis_app()
#' }
mdsvis_app <- function(preLoadDemoDataset = FALSE) {
    ui <- fluidPage(
        shinyjs::useShinyjs(),
        titlePanel("Plot of Metric MDS object"),
        tabsetPanel(
            id = "tabs",
            tabPanel("Input files",
                     sidebarLayout(
                         sidebarPanel(
                             fileInput("mdsObjFile", 
                                       "Choose MDS object file",
                                       accept = ".rds"),
                             fileInput("pDataFile", 
                                       "Choose phenodata file",
                                       accept = ".rds"),
                             uiOutput("pDataVariableSelector"),
                             fileInput("statsFile", 
                                       "Choose stats file",
                                       accept = ".rds")
                         ),
                         mainPanel()
                     )
            ),
            tabPanel("View",
                     sidebarLayout(
                         sidebarPanel(
                             selectInput("axis1", 
                                         "X-axis coordinate:", 
                                         choices = NULL),
                             selectInput("axis2", 
                                         "Y-axis coordinate:", 
                                         choices = NULL),
                             selectInput("colourBy", 
                                         "Colour by:", 
                                         choices = NULL),
                             conditionalPanel(
                                 condition =
                                     "input.colourBy != '_'",
                                 checkboxInput("ellipses",
                                              "Add ellipses",
                                              value = FALSE)
                             ),
                             selectInput("labelBy", 
                                         "Label by:", 
                                         choices = NULL),
                             selectInput("shapeBy", 
                                         "Shape by:", 
                                         choices = NULL),
                             selectInput("facetBy", 
                                         "Facet by:", 
                                         choices = NULL),
                             checkboxInput("biplot", 
                                           "Show biplot", 
                                           value = FALSE),
                             conditionalPanel(
                                 condition = "input.biplot == true",
                                 selectInput("extVariables", 
                                             "Select statistic:",
                                             choices = NULL),
                                 numericInput("arrowThreshold", 
                                              "Arrow length threshold",
                                              value = 0.8, 
                                              step = 0.1)
                             ),
                             checkboxInput("plotlytooltipping", 
                                           "Show plotly plot",
                                           value = FALSE),
                             conditionalPanel(
                                 condition = "input.plotlytooltipping == true",
                                 selectInput("pDataForAdditionalLabelling",
                                             "Additional plotly tooltips:",
                                             choices = NULL, 
                                             multiple = TRUE)
                             ),
                             checkboxInput("flipXAxis", 
                                           "Flip X-axis", 
                                           value = FALSE),
                             checkboxInput("flipYAxis", 
                                           "Flip Y-axis", 
                                           value = FALSE),
                             actionButton("exportPlot", 
                                          "Export plot as PDF")
                         ),
                         mainPanel(
                             plotOutput("mdsPlot"),
                             plotly::plotlyOutput("mdsPlotly")
                         )
                     )
            ),
            tabPanel("General Settings",
                     sidebarLayout(
                         sidebarPanel(
                             numericInput("pointLabelSize", 
                                          "Point label size",
                                          value = 3, step = 0.1),
                             checkboxInput("repelPointLabels", 
                                           "Repel overlapping labels",
                                           value = TRUE),
                             conditionalPanel(
                                 condition = "input.repelPointLabels == true",
                                 numericInput("maxOverlaps", 
                                              "Maximum overlaps", 
                                              value = 10,
                                              step = 1)
                             ),
                             checkboxInput("displayPseudoRSq", 
                                           "Display pseudo RSquare",
                                           value = TRUE),
                             checkboxInput("pointSizeReflectingStress",
                                           "Point size reflecting stress", 
                                           value = FALSE),
                             conditionalPanel(
                                 condition = 
                                     "input.pointSizeReflectingStress == false",
                                 numericInput("pointSize", 
                                              "Point size", 
                                              value = 1,
                                              step = 0.1)
                             ),
                             checkboxInput("displayArrowLabels", 
                                           "Display arrow labels",
                                           value = TRUE),
                             conditionalPanel(
                                 condition = "input.displayArrowLabels == true",
                                 numericInput("arrowLabelSize", 
                                              "Arrow label size",
                                              value = 3, step = 0.1),
                                 checkboxInput("repelArrowLabels", 
                                               "Repel arrow labels",
                                               value = FALSE)
                             ),
                             numericInput("ellipseLevel",
                                          "Ellipses level",
                                          value = 0.95,
                                          min = 0.01,
                                          max = 0.99,
                                          step = 0.01)
                         ),
                         mainPanel()
                     )
            ),
            selected = ifelse(preLoadDemoDataset, "View", "Input files")
        )
    )
    
    server <- function(input, output, session) {
        
        createMDSObjectFromFile <- function(mdsObjFilePath){
            mds <- NULL
            tryCatch({
                mds <- readRDS(mdsObjFilePath)
                if (!inherits(mds, "MDS")) {
                    rvIsMdsObjValid(FALSE)
                    mds <- NULL
                    stop("The selected file does not contain a MDS object.")
                }
                rvIsMdsObjValid(TRUE)
                
                updateSelectInput(session, "axis1",
                                  choices = seq_len(CytoMDS::nDim(mds)),
                                  selected = 1)
                updateSelectInput(session, "axis2",
                                  choices = seq_len(CytoMDS::nDim(mds)),
                                  selected = 2)
            }, error = function(e) {
                showNotification(as.character(e$message), 
                                 type = "error", 
                                 duration = NULL)
            })
            
            mds
        }
        
        createPDataFromFile <- function(pDataFilePath){
            
            pdata <- NULL
            tryCatch({
                pdata <- readRDS(pDataFilePath)
                
                if (!is.data.frame(pdata)) {
                    rvIsPDataValid(FALSE)
                    pdata <- NULL
                    stop("The selected file does not contain ",
                         "a data.frame object.")
                }
                
                rvIsPDataValid(TRUE)
                
                updateSelectInput(session, "colourBy",
                                  choices = c(
                                      "_", 
                                      colnames(pdata)
                                      [vapply(pdata,
                                              FUN = function(x){
                                                  is.factor(x) || 
                                                      is.numeric(x) ||
                                                      is.logical(x)
                                              },
                                              FUN.VALUE = logical(1))]),
                                  selected = "_")
                updateSelectInput(session, "labelBy",
                                  choices = c(
                                      "_", 
                                      colnames(pdata)),
                                  selected = "_")
                updateSelectInput(session, "shapeBy",
                                  choices = c(
                                      "_",
                                      colnames(pdata)
                                      [vapply(
                                          pdata,
                                          FUN = function(x){
                                              is.factor(x) && 
                                                  length(levels(x)) <= 6
                                          },
                                          FUN.VALUE = logical(1))]),
                                  selected = "_")
                updateSelectInput(session, "facetBy",
                                  choices = c(
                                      "_", 
                                      colnames(pdata)
                                      [vapply(
                                          pdata,
                                          FUN = is.factor,
                                          FUN.VALUE = logical(1))]),
                                  selected = "_")
                updateSelectInput(session, "pDataForAdditionalLabelling",
                                  choices = colnames(pdata))
                
                
                
            }, error = function(e) {
                showNotification(as.character(e$message), 
                                 type = "error", 
                                 duration = NULL)
            })
            
            pdata
        }
        
        createStatsFromFile <- function(statsFilePath) {
            stats <- NULL
            tryCatch({
                stats <- readRDS(statsFilePath)
                if (!(is.list(stats) && all(vapply(stats, 
                                                   FUN = is.matrix,
                                                   FUN.VALUE = logical(1))))) {
                    rvIsStatsValid(FALSE)
                    stats <- NULL
                    updateSelectInput(session, 
                                      "extVariables", 
                                      choices = c("_"))
                    stop("The selected file does not contain ",
                         "a list of matrices")
                }
                
                rvIsStatsValid(TRUE)
                
                updateSelectInput(session, "extVariables", 
                                  choices = names(stats),
                                  selected = names(stats)[1])
                
            }, error = function(e) {
                showNotification(as.character(e$message), 
                                 type = "error", 
                                 duration = NULL)
            })
            stats
        }
        
        checkInputObjectDims <- function(mds, pdata, stats) {
            dimCheck <- checkDimCompatibility(
                mds, pdata, stats)
            
            tryCatch({
                
                notNAdims <- dimCheck$dims[!is.na(dimCheck$dims)]
                if(length(unique(notNAdims)) > 1) {
                    stop("The loaded objects have incompatible ",
                         "number of samples (",
                         paste(
                             vapply(
                                 seq_along(notNAdims),
                                 FUN = function(i) paste(notNAdims[i],
                                                         names(notNAdims)[i],
                                                         sep = ": "),
                                 FUN.VALUE = character(1)),
                             collapse = ", "),
                         ")")
                }
            }, error = function(e) {
                showNotification(as.character(e$message), 
                                 type = "error", 
                                 duration = NULL)
            })
            
            dimCheck
        }
        
        
        # Increase the maximum upload size to 50 MB
        options(shiny.maxRequestSize = 50 * 1024^2)
        
        rvIsMdsObjValid <- reactiveVal(TRUE)
        rvIsPDataValid <- reactiveVal(TRUE)
        rvIsStatsValid <- reactiveVal(TRUE)
        rvAreMDSPdataCompatible <- reactiveVal(TRUE)
        rvAreMDSStatsCompatible <- reactiveVal(TRUE)
        
        
        if (preLoadDemoDataset) {
            initDataPath <- system.file("extdata", package = "MDSvis")
            initMDSFilePath <- file.path(initDataPath,
                                         "Krieg_mdsObj.rds")
            initPDataFilePath <- file.path(initDataPath,
                                           "Krieg_phenoData.rds")
            initStatFilePath <- file.path(initDataPath,
                                          "Krieg_chStats.rds")
            rvMdsObj <- reactiveVal(
                createMDSObjectFromFile(
                    mdsObjFilePath = initMDSFilePath)
            )
            
            myPData <- createPDataFromFile(initPDataFilePath)
            rvPData <- reactiveVal(myPData)      
            rvPDataSubs <- reactiveVal(myPData)
            
            rvStats <- reactiveVal(
                createStatsFromFile(initStatFilePath)
            )
        } else {
            rvMdsObj <- reactiveVal(NULL)
            rvPData <- reactiveVal(NULL)
            rvPDataSubs <- reactiveVal(NULL)
            rvStats <- reactiveVal(NULL)
        }
        
        observe({
            if (!rvIsMdsObjValid()) {
            #if (is.null(input$mdsObjFile) || !rvIsMdsObjValid()) {
                shinyjs::disable("axis1")
                shinyjs::disable("axis2")
                shinyjs::disable("colourBy")
                shinyjs::disable("ellipses")
                shinyjs::disable("labelBy")
                shinyjs::disable("facetBy")
                shinyjs::disable("shapeBy")
                shinyjs::disable("biplot")
                shinyjs::disable("plotlytooltipping")
                shinyjs::disable("flipXAxis")
                shinyjs::disable("flipYAxis")
                shinyjs::disable("pointLabelSize")
                shinyjs::disable("displayPointLabels")
                shinyjs::disable("repelPointLabels")
                shinyjs::disable("displayPseudoRSq")
                shinyjs::disable("pointSize")
                shinyjs::disable("pointSizeReflectingStress")
                shinyjs::disable("displayArrowLabels")
                shinyjs::disable("arrowLabelSize")
                shinyjs::disable("repelArrowLabels")
                shinyjs::disable("maxOverlaps")
                shinyjs::disable("exportPlot")
                if (input$repelPointLabels) {
                    shinyjs::disable("maxOverlaps")
                }
                if (!input$pointSizeReflectingStress) {
                    shinyjs::disable("pointSize")
                }
                if (input$displayArrowLabels) {
                    shinyjs::disable("arrowLabelSize")
                    shinyjs::disable("repelArrowLabels")
                } 
                shinyjs::disable("ellipseLevel")
                
            } 
            if (rvIsMdsObjValid()){
            #if (!is.null(input$mdsObjFile) && rvIsMdsObjValid()){
                shinyjs::enable("axis1")
                shinyjs::enable("axis2")
                shinyjs::enable("flipXAxis")
                shinyjs::enable("flipYAxis")
                shinyjs::enable("pointLabelSize")
                shinyjs::enable("displayPointLabels")
                shinyjs::enable("repelPointLabels")
                shinyjs::enable("displayPseudoRSq")
                shinyjs::enable("pointSize")
                shinyjs::enable("pointSizeReflectingStress")
                shinyjs::enable("plotlytooltipping")
                shinyjs::enable("maxOverlaps")
                shinyjs::enable("exportPlot")
                if (input$repelPointLabels) {
                    shinyjs::enable("maxOverlaps")
                }
                if (!input$pointSizeReflectingStress) {
                    shinyjs::enable("pointSize")
                }
                if (input$displayArrowLabels) {
                    shinyjs::enable("arrowLabelSize")
                    shinyjs::enable("repelArrowLabels")
                }
                shinyjs::enable("ellipseLevel")
                if (!is.null(rvStats()) && rvIsStatsValid() && 
                    rvAreMDSStatsCompatible()) {
                    shinyjs::enable("biplot")
                    shinyjs::enable("displayArrowLabels")
                    shinyjs::enable("arrowLabelSize")
                    shinyjs::enable("repelArrowLabels")
                    if (input$biplot) {
                        shinyjs::enable("extVariables")
                        shinyjs::enable("arrowThreshold")
                    }
                } else {
                    shinyjs::disable("biplot")
                    shinyjs::disable("displayArrowLabels")
                    shinyjs::disable("arrowLabelSize")
                    shinyjs::disable("repelArrowLabels")
                    if (input$biplot) {
                        shinyjs::disable("extVariables")
                        shinyjs::disable("arrowThreshold")
                    }
                }
                if (!is.null(rvPData()) && rvIsPDataValid() && 
                    rvAreMDSPdataCompatible()){
                    shinyjs::enable("colourBy")
                    shinyjs::enable("ellipses")
                    shinyjs::enable("labelBy")
                    shinyjs::enable("shapeBy")
                    shinyjs::enable("facetBy")
                    if(input$plotlytooltipping) {
                        shinyjs::enable("pDataForAdditionalLabelling")
                    }
                } else {
                    shinyjs::disable("colourBy")
                    shinyjs::disable("ellipses")
                    shinyjs::disable("labelBy")
                    shinyjs::disable("shapeBy")
                    shinyjs::disable("facetBy")
                    if(input$plotlytooltipping) {
                        shinyjs::disable("pDataForAdditionalLabelling")
                    }
                }
            }
        })
        
        observeEvent(input$mdsObjFile, {
            req(input$mdsObjFile)
            mds <- createMDSObjectFromFile(
                mdsObjFilePath = input$mdsObjFile$datapath)
            
            rvMdsObj(mds)
            
            dimCheck <- checkInputObjectDims(
                rvMdsObj(), rvPData(), rvStats())
            
            rvAreMDSPdataCompatible(
                dimCheck$dimCompatibility$areMDSPdataCompatible)
            rvAreMDSStatsCompatible(
                dimCheck$dimCompatibility$areMDSStatsCompatible)
        })
        
        observeEvent(input$pDataFile, {
            req(input$pDataFile)
            pdata <- createPDataFromFile(
                pDataFilePath = input$pDataFile$datapath)
            rvPData(pdata)
            rvPDataSubs(pdata)

            dimCheck <- checkInputObjectDims(
                rvMdsObj(), rvPData(), rvStats())

            
            rvAreMDSPdataCompatible(
                dimCheck$dimCompatibility$areMDSPdataCompatible)
        })
        
        observeEvent(input$statsFile, {
            req(input$statsFile)
            stats <- createStatsFromFile(
                statsFilePath = input$statsFile$datapath)
            rvStats(stats)
            
            dimCheck <- checkInputObjectDims(
                rvMdsObj(), rvPData(), rvStats())
            
            rvAreMDSStatsCompatible(
                dimCheck$dimCompatibility$areMDSStatsCompatible)
        })
        
        
        p <- reactive({
            if (!is.null(rvMdsObj()) && rvIsMdsObjValid() && 
                input$axis1 != "" && input$axis2 != "") {
                plotargs <- list(
                    mdsObj = rvMdsObj(),
                    projectionAxes = c(as.integer(input$axis1), 
                                       as.integer(input$axis2)),
                    flipXAxis = input$flipXAxis,
                    flipYAxis = input$flipYAxis,
                    pointLabelSize = input$pointLabelSize,
                    displayPointLabels = FALSE,
                    repelPointLabels = input$repelPointLabels,
                    displayPseudoRSq = input$displayPseudoRSq,
                    pointSizeReflectingStress = input$pointSizeReflectingStress,
                    pointSize = input$pointSize
                )
                if (!is.null(input$maxOverlaps)) {
                    plotargs$max.overlaps <- input$maxOverlaps
                }
                if (!is.null(rvStats()) && rvIsStatsValid() && 
                    rvAreMDSStatsCompatible()) {
                    plotargs$biplot <- input$biplot
                    plotargs$extVariables <- rvStats()[[input$extVariables]]
                    plotargs$displayArrowLabels <- input$displayArrowLabels
                    plotargs$arrowLabelSize <- input$arrowLabelSize
                    plotargs$repelArrowLabels <- input$repelArrowLabels
                    plotargs$arrowThreshold <- input$arrowThreshold
                }
                if (!is.null(rvPData()) && rvIsPDataValid() && 
                    rvAreMDSPdataCompatible()) {
                    plotargs$pData <- rvPDataSubs()
                    if (input$colourBy != "_") {
                        plotargs$pDataForColour <- input$colourBy
                    }
                    if (input$labelBy != "_") {
                        plotargs$pDataForLabel <- input$labelBy
                        plotargs$displayPointLabels <- TRUE
                    }
                    if (input$shapeBy != "_") {
                        plotargs$pDataForShape <- input$shapeBy
                    }
                }
                if (length(input$pDataForAdditionalLabelling)) {
                    plotargs$pDataForAdditionalLabelling <- 
                        input$pDataForAdditionalLabelling
                }
                pltargs <- 
                    plotargs[!(names(plotargs) 
                               %in% "pDataForAdditionalLabelling")]
                pltlyargs <- plotargs[!(names(plotargs) %in% "biplot")]
                pltlyargs$repelPointLabels <- FALSE
                plt <- do.call(CytoMDS::ggplotSampleMDS, pltargs)
                pltly <- do.call(CytoMDS::ggplotSampleMDS, pltlyargs)
                if (!is.null(rvPData()) && rvIsPDataValid() && 
                    rvAreMDSPdataCompatible() && input$facetBy != "_") {
                    plt <- plt + ggplot2::facet_wrap(~ .data[[input$facetBy]])
                    pltly <- pltly + 
                        ggplot2::facet_wrap(~ .data[[input$facetBy]])
                }
                if (!is.null(rvPData()) && rvIsPDataValid() &&
                    rvAreMDSPdataCompatible() && input$ellipses) {
                    plt <- plt +
                        ggplot2::stat_ellipse(
                            mapping = ggplot2::aes(
                                col = .data[[input$colourBy]]),
                            level = input$ellipseLevel)
                }
                list(plt = plt,
                     pltly = pltly)
            }
        })
        
        output$mdsPlot <- renderPlot({
            req(rvMdsObj())
            p()$plt
        })
        
        output$mdsPlotly <- plotly::renderPlotly({
            req(rvMdsObj())
            req(input$plotlytooltipping)
            plotly::ggplotly(p()$pltly)
        })
        
        output$pDataVariableSelector <- renderUI({
            req(rvPData())
            selectInput("pDataVariableSelection",
                        "Select phenodata variables to include:",
                        choices = colnames(rvPData()),
                        selected = colnames(rvPData()),
                        multiple = TRUE)
        })
        
        observeEvent(input$pDataVariableSelection, {
            req(rvPData(), input$pDataVariableSelection)
            rvPDataSubs(rvPData()[, input$pDataVariableSelection, drop = FALSE])
            updateSelectInput(session, "colourBy",
                              choices = c(
                                  "_", 
                                  colnames(rvPDataSubs())[
                                      vapply(
                                          rvPDataSubs(),
                                          FUN = function(x){
                                              is.factor(x) || 
                                                  is.numeric(x) ||
                                                  is.logical(x)
                                          },
                                          FUN.VALUE = logical(1))]),
                              selected = "_")
            updateSelectInput(session, "labelBy",
                              choices = c("_", 
                                          colnames(rvPDataSubs())),
                              selected = "_")
            updateSelectInput(session, "shapeBy",
                              choices = c(
                                  "_",
                                  colnames(rvPDataSubs())
                                  [vapply(
                                      rvPDataSubs(),
                                      FUN = function(x){
                                          is.factor(x) && 
                                              length(levels(x)) <= 6
                                      },
                                      FUN.VALUE = logical(1))]),
                              selected = "_")
            updateSelectInput(session, "facetBy",
                              choices = c(
                                  "_",
                                  colnames(rvPDataSubs())
                                  [vapply(
                                      rvPDataSubs(),
                                      FUN = is.factor,
                                      FUN.VALUE = logical(1))]),
                              selected = "_")
            updateSelectInput(session, "pDataForAdditionalLabelling",
                              choices = colnames(rvPDataSubs()))
        })
        
        observeEvent(input$exportPlot, {
            showModal(modalDialog(
                title = "Export Settings",
                textInput("plot_title", 
                          "Plot title", 
                          value = "Multi Dimensional Scaling"),
                numericInput("plot_width", 
                             "Width (inches)", 
                             value = 7, 
                             min = 1),
                numericInput("plot_height", "Height (inches)", 
                             value = 5, 
                             min = 1),
                downloadButton("confirmDownload", 
                               "Download PDF"),
                easyClose = TRUE
            ))
        })
        
        output$confirmDownload <- downloadHandler(
            filename = function() { "Multi Dimensional Scaling.pdf" },
            
            content = function(file) {
                
                removeModal()
                
                title <- input$plot_title
                width <- input$plot_width
                height <- input$plot_height
                
                validate(
                    need(!is.null(width) && !is.null(height), 
                         "Width and height must be specified.")
                )
                
                ggplot2::ggsave(file, 
                                plot = p()$plt + ggplot2::labs(title = title), 
                                device = "pdf", 
                                width = width, 
                                height = height)
            }
        )
        
    }
    
    shinyApp(ui = ui, server = server)
}

# MDSvis - Copyright (C) <2025-2026>
# <Université catholique de Louvain (UCLouvain), Belgique>
#
#   Description and complete License: see LICENSE file.
#
# This program (MDSvis) is free software:
#   you can redistribute it and/or modify it under the terms of the GNU General
# Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details (<http://www.gnu.org/licenses/>).
#

# This file is part of the standard setup for testthat.
# It is recommended that you do not modify it.
#
# Where should you do additional test configuration?
# Learn more about the roles of various files in:
# * https://r-pkgs.org/testing-design.html#sec-tests-files-overview
# * https://testthat.r-lib.org/articles/special-files.html

library(testthat)
library(MDSvis)

test_check("MDSvis")

## ----style, echo=FALSE, results='hide', message=FALSE-------------------------
BiocStyle::markdown()

## ----package_install, eval=FALSE, message=FALSE, warning=FALSE----------------
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# if (!require("MDSvis", quietly = TRUE))
#     BiocManager::install("MDSvis")

## ----rlibs, results=FALSE, message=FALSE, warning=FALSE-----------------------
library(HDCytoData)
library(CytoMDS)
library(MDSvis)

## ----loadDataSet--------------------------------------------------------------
Krieg_fs <- Krieg_Anti_PD_1_flowSet()
Krieg_fs

## ----convertPhenoData---------------------------------------------------------
# update phenoData structure

chLabels <- 
    keyword(Krieg_fs[[1]], "MARKER_INFO")$MARKER_INFO$channel_name
chMarkers <- 
    keyword(Krieg_fs[[1]], "MARKER_INFO")$MARKER_INFO$marker_name
marker_class <- 
    keyword(Krieg_fs[[1]], "MARKER_INFO")$MARKER_INFO$marker_class

chLabels <- chLabels[marker_class != "none"]
chMarkers <- chMarkers[marker_class != "none"]
# marker_class all equal to "type", 24 markers are left

phenoData <- flowCore::pData(Krieg_fs)
additionalPhenoData <- 
    keyword(Krieg_fs[[1]], "EXPERIMENT_INFO")$EXPERIMENT_INFO
phenoData <- cbind(phenoData, additionalPhenoData)
    
flowCore::pData(Krieg_fs) <- phenoData

## ----distCalc-----------------------------------------------------------------
# transform flow set (arcsinh(cofactor = 5))
trans <- arcsinhTransform(
    transformationId="ArcsinhTransform", 
    a = 0, 
    b = 1/5, 
    c = 0)

# Applying arcsinh() transformation
Krieg_fs_trans <- transform(
    Krieg_fs,
    transformList(chLabels, trans))

# Calculating Sample distances
pwDist <- pairwiseEMDDist(
    Krieg_fs_trans,
    channels = chMarkers,
    verbose = FALSE
)

## ----MDSCalc------------------------------------------------------------------
mdsObj <- computeMetricMDS(
    pwDist,
    seed = 0)

show(mdsObj)

## ----statsCalc----------------------------------------------------------------
# Computing sample statistics

statFUNs <- c(
    "median" = stats::median, 
    "std-dev" = stats::sd,
    "mean" = base::mean,
    "Q25" = function(x, na.rm) 
        stats::quantile(x, probs = 0.25, na.rm = na.rm),
    "Q75" = function(x, na.rm) 
        stats::quantile(x, probs = 0.75, na.rm = na.rm))

chStats <- CytoMDS::channelSummaryStats(
    Krieg_fs_trans,
    channels = chMarkers,
    statFUNs = statFUNs)


## ----saveRDS, eval=FALSE------------------------------------------------------
# saveRDS(mdsObj, file = "Krieg_mdsObj.rds")
# saveRDS(phenoData, file = "Krieg_phenoData.rds")
# saveRDS(chStats, file = "Krieg_chStats.rds")

## ----launch, eval = FALSE-----------------------------------------------------
# MDSvis::mdsvis_app()
# 
# # alternatively launch the application in demo mode, with the Krieg_Anti_PD_1
# # dataset already loaded.
# #MDSvis::mdsvis_app(preLoadDemoDataset = TRUE)
# 

## ----sessioninfo, echo=FALSE--------------------------------------------------
sessionInfo()

# MDSvis - Copyright (C) <2025-2026>
# <Université catholique de Louvain (UCLouvain), Belgique>
#
#   Description and complete License: see LICENSE file.
#
# This program (MDSvis) is free software:
#   you can redistribute it and/or modify it under the terms of the GNU General
# Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details (<http://www.gnu.org/licenses/>).

#' @title MDSvis package
#' 
#' @name MDSvis
#'
#' @rdname MDSvis
#' 
#' @seealso \link[CytoMDS:CytoMDS-package]{CytoMDS}
#' @seealso [MDSvis::mdsvis_app]
#'
#' @description
#'
#' `MDSvis` provides a shiny application for interactive visualisation of 
#' Multi Dimensional Scaling (MDS) results.

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL

## ----style, echo=FALSE, results='hide', message=FALSE-------------------------
BiocStyle::markdown()

## ----package_install, eval=FALSE, message=FALSE, warning=FALSE----------------
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# if (!require("MDSvis", quietly = TRUE))
#     BiocManager::install("MDSvis")

## ----rlibs, results=FALSE, message=FALSE, warning=FALSE-----------------------
library(CytoMDS)
library(MDSvis)

## ----loadDataSet--------------------------------------------------------------
# inputs
inputDir <- system.file("extdata", package = "MDSvis")
distPath <- file.path(inputDir, "fsap_mature_226_distances.txt")
propPath <- file.path(inputDir, "fsap_mature_226_properties.txt")

# reading distance matrix from disk
distData <- read.delim(distPath)
rownames(distData) <- distData$X
distData$X <- NULL
distMat <- as.matrix(distData)

# reading properties
propDF <- read.delim(propPath)

## ----convertPhenoData---------------------------------------------------------
# split properties between factors and continuous variables (stats)

statMat <- as.matrix(propDF[, -1])
statList <- list(quant_properties = statMat)

propDF <- propDF[, 1, drop = FALSE]
propDF$family <- factor(substr(propDF$Sequence_Name, start = 1, stop = 3))
propDF$genus <- factor(substr(propDF$Sequence_Name, start = 1, stop = 6))
propDF$species <- factor(substr(propDF$Sequence_Name, start = 1, stop = 8))

## ----tables-------------------------------------------------------------------
table(propDF$family)
table(propDF$genus)
table(propDF$species)

## ----MDSCalc------------------------------------------------------------------
MDSObj <- CytoMDS::computeMetricMDS(pwDist = distMat, 
                                    nDim = 10)

show(MDSObj)

## ----saveRDS, eval=FALSE------------------------------------------------------
# # outputs (for shiny app)
# MDSRDSPath <- "fsap_mature_MDS.rds"
# statRDSPath <- "fsap_mature_stats.rds"
# propRDSPath <- "fsap_mature_props.rds"
# 
# # saving objects
# saveRDS(MDSObj, file = MDSRDSPath)
# saveRDS(statList, file = statRDSPath)
# saveRDS(propDF, file = propRDSPath)

## ----launch,eval=FALSE--------------------------------------------------------
# MDSvis::mdsvis_app()

## ----sessioninfo, echo=FALSE--------------------------------------------------
sessionInfo()

# MDSvis - Copyright (C) <2025-2026>
# <Université catholique de Louvain (UCLouvain), Belgique>
#
#   Description and complete License: see LICENSE file.
#
# This program (MDSvis) is free software:
#   you can redistribute it and/or modify it under the terms of the GNU General
# Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details (<http://www.gnu.org/licenses/>).
#

# checks compatibility of dimensions of the three input objects
#' @importFrom methods is
checkDimCompatibility <- function(
        mdsObj, pData, stats) {
    dims <- rep(NA, 3)
    names(dims) <- c("MDS", "phenodata", "stats")
    
    if (is(mdsObj, "MDS")) {
        dims[1] <- CytoMDS::nPoints(mdsObj)
    }
    
    if (!is.null(pData)) {
        dims[2] <- nrow(pData)
    }
    
    if (!is.null(stats)) {
        dims[3] <- nrow(stats[[1]])
    }
    
    dimCompatibility <- list(
            areMDSPdataCompatible = TRUE,
            areMDSStatsCompatible = TRUE)
    
    if (!is.na(dims[1])) {
        if (!is.na(dims[2]) && dims[1] != dims[2]) {
            dimCompatibility$areMDSPdataCompatible <- FALSE
        }
            
        if (!is.na(dims[3]) && dims[1] != dims[3]) {
            dimCompatibility$areMDSStatsCompatible <- FALSE
        }
            
    }
    dimCheck <- list(
        dims = dims,
        dimCompatibility = dimCompatibility
    )
    dimCheck
}
    
    
    

# MDSvis - Copyright (C) <2025-2026>
# <Université catholique de Louvain (UCLouvain), Belgique>
#
#   Description and complete License: see LICENSE file.
#
# This program (MDSvis) is free software:
#   you can redistribute it and/or modify it under the terms of the GNU General
# Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details (<http://www.gnu.org/licenses/>).
#

library(CytoMDS)

# load objects from text fixtures
Krieg_mdsObj <- readRDS(test_path("fixtures", "Krieg_mdsObj.rds"))
Krieg_phenoData <- readRDS(test_path("fixtures", "Krieg_phenoData.rds"))
Krieg_chStats <- readRDS(test_path("fixtures", "Krieg_chStats.rds"))

test_that("checkDimCompatibility works", {
    
    # start with initial objects
    nSamples = nrow(Krieg_phenoData)
    
    ret <- checkDimCompatibility(
        mdsObj = Krieg_mdsObj, 
        pData = Krieg_phenoData, 
        stats = Krieg_chStats
    )
    
    expect_equal(length(ret$dims), 3)
    expect_equal(names(ret$dims), c("MDS", "phenodata", "stats"))
    expect_equal(unname(ret$dims[1]), nSamples)
    expect_equal(unname(ret$dims[2]), nSamples)
    expect_equal(unname(ret$dims[3]), nSamples)
    expect_type(ret$dimCompatibility, "list")
    expect_equal(length(ret$dimCompatibility), 2)
    expect_equal(
        names(ret$dimCompatibility), 
        c("areMDSPdataCompatible", "areMDSStatsCompatible"))
    expect_equal(ret$dimCompatibility[[1]], TRUE)
    expect_equal(ret$dimCompatibility[[2]], TRUE)
    
    # now perturb pData
    pDataWrong <- Krieg_phenoData[-1,]
    ret2 <- checkDimCompatibility(
        mdsObj = Krieg_mdsObj, 
        pData = pDataWrong, 
        stats = Krieg_chStats
    )    
    expect_equal(length(ret2$dims), 3)
    expect_equal(names(ret2$dims), c("MDS", "phenodata", "stats"))
    expect_equal(unname(ret2$dims[1]), nSamples)
    expect_equal(unname(ret2$dims[2]), nSamples-1)
    expect_equal(unname(ret2$dims[3]), nSamples)
    expect_type(ret2$dimCompatibility, "list")
    expect_equal(length(ret2$dimCompatibility), 2)
    expect_equal(
        names(ret2$dimCompatibility), 
        c("areMDSPdataCompatible", "areMDSStatsCompatible"))
    expect_equal(ret2$dimCompatibility[[1]], FALSE)
    expect_equal(ret2$dimCompatibility[[2]], TRUE)
    
    # now perturb stats
    statsWrong <- lapply(
        Krieg_chStats,
        FUN = function(x) x[-1,]
    )
    
    ret3 <- checkDimCompatibility(
        mdsObj = Krieg_mdsObj, 
        pData = Krieg_phenoData, 
        stats = statsWrong
    )
    
    expect_equal(length(ret3$dims), 3)
    expect_equal(names(ret3$dims), c("MDS", "phenodata", "stats"))
    expect_equal(unname(ret3$dims[1]), nSamples)
    expect_equal(unname(ret3$dims[2]), nSamples)
    expect_equal(unname(ret3$dims[3]), nSamples-1)
    expect_type(ret3$dimCompatibility, "list")
    expect_equal(length(ret3$dimCompatibility), 2)
    expect_equal(
        names(ret3$dimCompatibility), 
        c("areMDSPdataCompatible", "areMDSStatsCompatible"))
    expect_equal(ret3$dimCompatibility[[1]], TRUE)
    expect_equal(ret3$dimCompatibility[[2]], FALSE)
})

test_that("checkDimCompatibility with edge cases works", {
    dummyMDS <- 3
    dummyPData <- NULL
    dummyStats <- NULL
    
    ret <- checkDimCompatibility(
        mdsObj = dummyMDS, 
        pData = dummyPData, 
        stats = dummyStats
    )
    
    expect_equal(length(ret$dims), 3)
    expect_equal(names(ret$dims), c("MDS", "phenodata", "stats"))
    expect_equal(unname(ret$dims[1]), NA)
    expect_equal(unname(ret$dims[2]), NA)
    expect_equal(unname(ret$dims[3]), NA)
    expect_type(ret$dimCompatibility, "list")
    expect_equal(length(ret$dimCompatibility), 2)
    expect_equal(
        names(ret$dimCompatibility), 
        c("areMDSPdataCompatible", "areMDSStatsCompatible"))
    expect_equal(ret$dimCompatibility[[1]], TRUE)
    expect_equal(ret$dimCompatibility[[2]], TRUE)
})

---
title: "Preparing `MDSvis` input objects from a distance matrix and sample properties"
author:
    - name: Philippe Hauchamps
    - name: Shabnam Zaman
package: MDSvis
abstract: >
 This vignette shows an example of building input objects for using `MDSvis`.  
 Here the starting point is: (1) a pairwise distance matrix, where each row 
 and each column correspond to one sample, and (2) additional sample specific 
 properties.  
 This vignette is distributed under a CC BY-SA 4.0 license.
output:
  BiocStyle::html_document:
    toc_float: true
bibliography: MDSvis.bib
vignette: >
  %\VignetteIndexEntry{Preparing input objects from a distance matrix and sample properties}
  %\VignetteEngine{knitr::rmarkdown}
  %%\VignetteKeywords{multidimensionalScaling, Software, Visualization}
  %\VignetteEncoding{UTF-8}
---

```{r style, echo=FALSE, results='hide', message=FALSE}
BiocStyle::markdown()
```

# Introduction

This short vignette is complementary to the [main](MDSvis.html) `MDSvis` package 
vignette, which provides an overview of `MDSvis` Shiny application (app) 
features.  

Here we focus on 'how to' create some input objects, which need to be provided 
to the Shiny app. In this vignette, we show that we can take advantage of the 
*Multi Dimensional Scaling (MDS)* interactive visualisations, 
starting from any distance matrix, and some sample specific properties.  

# Installation and loading dependencies

To install this package, start R and enter (un-commented):

```{r package_install, eval=FALSE, message=FALSE, warning=FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

if (!require("MDSvis", quietly = TRUE))
    BiocManager::install("MDSvis")
```

We now load the packages needed in the current vignette:

```{r rlibs, results=FALSE, message=FALSE, warning=FALSE}
library(CytoMDS)
library(MDSvis)
```


# Dataset

For this vignette, we use the *frog skin active peptide (fsap)* dataset. 
It consists of 226 samples, where each sample represents an amino acid sequence 
corresponding to a reported amphibian host defense peptide.

Distances between these peptides can be obtained using pairwise alignments and 
computing an appropriate dissimilarity metric between their amino acid 
sequences. For details, see the [fsap dataset methods section](#fsap_methods) 
below.   

On top of the peptide distances, peptide properties have also been collected:  
- Uniprot accession number  
- taxonomic family, genus, species  
- some physicochemical parameters  

In the following code, we import the two text files of the dataset:  
- the distance matrix file (imported as a R `matrix`)  
- the sample properties file (imported as a R `data.frame`)  

```{r loadDataSet}
# inputs
inputDir <- system.file("extdata", package = "MDSvis")
distPath <- file.path(inputDir, "fsap_mature_226_distances.txt")
propPath <- file.path(inputDir, "fsap_mature_226_properties.txt")

# reading distance matrix from disk
distData <- read.delim(distPath)
rownames(distData) <- distData$X
distData$X <- NULL
distMat <- as.matrix(distData)

# reading properties
propDF <- read.delim(propPath)
```

# Generation of input files for the Shiny application

We still need to pre-process the data properties and split them into two parts:
- the variables that will be used to annotate the plots, or associate colour,
shape, faceting etc. These variable are mostly factor variables. We extract 
them into the *phenoData* object - which needs to be a `data.frame`.
- the continuous variables that can be exploited as sample *statistics* in the 
bi-plots. These need to be included in a list of statistics object.

```{r convertPhenoData}
# split properties between factors and continuous variables (stats)

statMat <- as.matrix(propDF[, -1])
statList <- list(quant_properties = statMat)

propDF <- propDF[, 1, drop = FALSE]
propDF$family <- factor(substr(propDF$Sequence_Name, start = 1, stop = 3))
propDF$genus <- factor(substr(propDF$Sequence_Name, start = 1, stop = 6))
propDF$species <- factor(substr(propDF$Sequence_Name, start = 1, stop = 8))
```

Below are frequency tables for peptides belonging to the different families, 
genera and species.

```{r tables}
table(propDF$family)
table(propDF$genus)
table(propDF$species)
```

From the distance matrix, we now need to calculate the MDS projection object, 
which will be used as main input in the Shiny app.  
This is done using the `CytoMDS` package. Here we set the number of projection 
dimensions to 10. See the documentation of `CytoMDS::computeMetricMDS()` for 
details on other possible options.

```{r MDSCalc}
MDSObj <- CytoMDS::computeMetricMDS(pwDist = distMat, 
                                    nDim = 10)

show(MDSObj)
```

The newly created objects can be saved as .rds files.  
These files can in turn be selected within the shiny app for visualization.   

```{r saveRDS, eval=FALSE}
# outputs (for shiny app)
MDSRDSPath <- "fsap_mature_MDS.rds"
statRDSPath <- "fsap_mature_stats.rds"
propRDSPath <- "fsap_mature_props.rds"

# saving objects 
saveRDS(MDSObj, file = MDSRDSPath)
saveRDS(statList, file = statRDSPath)
saveRDS(propDF, file = propRDSPath)
```

# Visualization of the MDS projection

The `MDSvis` function `run_app` launches the interactive Shiny app. 
From here, you can take a look at the [main](MDSvis.html) `MDSvis` vignette 
for an overview of the Shiny app features.   

```{r launch,eval=FALSE}
MDSvis::mdsvis_app()
```

Here below is a screen shot from the Shiny app, 
displaying a view of the MDS projection of the `fsap` dataset.

![](images/fsap_view.png)


# fsap dataset methods
<a name="fsap_methods">
</a>


## Data mining and assembly of a reference dataset  

For the construction of an initial dataset 
of known amphibian host defense peptides, sequences belonging to 
the “frog skin active peptide” superfamily (FSAP; InterPro: IPR016322) 
were collected from the Uniprot database and relevant literature. 
FSAP from the higher-level taxonomic classifications of Ranidae, 
Microhylidae and Hyloidea are differentiated using the prefixes RAN_, MIC_ 
and HYL_, respectively. These designations are broadly referred to as "family" 
throughout this vignette. Each family prefix is followed by a letter code 
to identify the genus and species from which the FSAP was identified, 
followed by its unique Uniprot accession. 
Each FSAP precursor protein is composed of a tripartite structure 
in which the N-terminal signal peptide and spacer regions 
are enzymatically cleaved to release the bioactive, or "mature" peptide. 
A total of 226 mature FSAP were thus identified and used 
for subsequent analysis.

## Pairwise alignments using primary structure and hydrophobicity  

Pairwise alignments of mature FSAP sequences were generated 
with AlignMe 1.2 [@Stamm2013-kz] using a combined scale 
that incorporates the Hessa, White and von Heinje hydropathy scale 
(HWvH; weight: 2.6 [@Hessa2007-rc] and the VTML substitution matrix 
(weight: 0.5), according to the developer’s recommendations. 
Gap penalties were similarly based on [@Stamm2014-wp] and were set as follows: 
below threshold gap opening penalty: 18.0, 
below threshold gap extension penalty: 1.3, 
above threshold gap opening penalty: 11.0, 
above threshold gap extension penalty 4.4: 
thresholds for penalties: 0; 
termini gap opening penalty 13.1; 
termini gap extension penalty: 0.9 [@Stamm2014-wp]. 
Given the short lengths of mature FSAP sequences, 
window size was set to 0 to prevent a smoothing effect. 
Absolute differences between the hydrophobicity measurements generated 
for each peptide pair were calculated and used to construct a distance, 
or dissimilarity matrix.

## Physicochemical parameters distinguishing FSAP  

The following parameters were calculated for each mature FSAP sequence:  
1. peptide length (number of amino acids)  
2. grand average of hydropathy, or GRAVY index (obtained by dividing 
the sum of hydropathy values of all amino acids by total sequence length), 
to determine the hydrophobicity/hydrophilicity of the protein; 
a positive index indicates a more hydrophobic character, 
whereas a negative index indicates a more hydrophilic character  
3. hydrophobic moment (measure of the amphipathicity of a structure, 
or asymmetry of hydrophobicity, by quantifying 
the distribution of hydrophobic and hydrophilic residues along its length; 
particularly relevant for α-helices)  
4. isoelectric point, or pI (pH at which the peptide carries 
a net electrical charge of zero)  
5. molar mass  
6. net charge at pH 7  
7. cysteine content (%)  
8. ratio of lysines to arginines (e.g., changes in this ratio in α-helical 
peptides can affect the hydrophobicity required for membrane perturbation)  


# Acknowledgement
We would like to thank Kim Roelants for devising the method 
to calculate distances between FSAP, 
as described in the [fsap dataset methods section](#fsap_methods) 
in the current vignette. 


# Session information {-}

```{r sessioninfo, echo=FALSE}
sessionInfo()
```

# References {-}

---
title: "Visualization of Multi Dimensional Scaling (MDS) objects"
author:
    - name: Andrea Vicini
    - name: Philippe Hauchamps
package: MDSvis
abstract: >
 This vignette is the main one describing the functionality implemented 
 in the `MDSvis` package.  
 `MDSvis` conatins a shiny application enabling interactive visualization of 
 Multi Dimensional Scaling (MDS) objects.  
 This vignette is distributed under a CC BY-SA 4.0 license.
output:
  BiocStyle::html_document:
    toc_float: true
bibliography: MDSvis.bib
vignette: >
  %\VignetteIndexEntry{Visualization of Multi Dimensional Scaling (MDS) objects}
  %\VignetteEngine{knitr::rmarkdown}
  %%\VignetteKeywords{multidimensionalScaling, Software, Visualization}
  %\VignetteEncoding{UTF-8}
---

```{r style, echo=FALSE, results='hide', message=FALSE}
BiocStyle::markdown()
```

# Installation and loading dependencies

To install this package, start R and enter:

```{r package_install, eval=FALSE, message=FALSE, warning=FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

if (!require("MDSvis", quietly = TRUE))
    BiocManager::install("MDSvis")
```

We now load the packages needed in the current vignette:

```{r rlibs, results=FALSE, message=FALSE, warning=FALSE}
library(HDCytoData)
library(CytoMDS)
library(MDSvis)
```

# Introduction

The `MDSvis` package implements visualization of Multi Dimensional Scaling (MDS)
objects representing a low dimensional projection of sample distances.
Such objects can be obtained using features implemented in the `CytoMDS` 
package (@Hauchamps2025-zm). For more information see the [`CytoMDS` vignette](https://www.bioconductor.org/packages/release/bioc/vignettes/CytoMDS/inst/doc/CytoMDS.html).  

The visualization is done via a Shiny app that allows 
the user to interactively customise the plots depending on a series of input
parameters (see the `CytoMDS::ggplotSampleMDS()` function for more details 
on the parameters).  

**IMPORTANT**: the example provided in the current vignette uses cytometry data. 
As a result, the distance matrix, which contains the pairwise sample distances, 
and serves as input to the MDS projection, is here 
calculated from cytometry data samples using the `CytoMDS` package. 
However, the input  MDS object can also be calculated from any distance matrix, 
where the units of interest are not necessarily cytometry samples. This is 
demonstrated in this additional [vignette](MDSvis_Input_From_Distance.html).

# Illustrative dataset

We load an illustrative mass cytometry (*CyTOF*) dataset from
[@Krieg2018-zs], accessible from the Bioconductor *HDCytoData* data
package (@Weber2019-qp).   

*Krieg_Anti_PD_1* was used to characterize immune cell subsets in 
peripheral blood from melanoma skin cancer patients treated 
with anti-PD-1 immunotherapy.   

This study found that the frequency of CD14+CD16-HLA-DRhi monocytes 
in baseline samples (taken from patients prior to treatment) was a strong 
predictor of survival in response to immunotherapy treatment.  

Notably, this dataset contains a strong batch effect, 
due to sample acquisition on two different days (@Krieg2018-zs).

# Generation of input files

In the current section, we show how to build the input objects needed 
by the shiny app, step by step. However, if you are only interested 
in the description of the app functionalities, you might decide to launch 
the application demo mode - which automatically pre-loads 
the *Krieg_Anti_PD_1* dataset - and jump to 
[the next section](#visualization-of-the-mds-projection). 

First we load the *Krieg_Anti_PD_1* dataset, from the *HDCytoData* package.  

```{r loadDataSet}
Krieg_fs <- Krieg_Anti_PD_1_flowSet()
Krieg_fs
```

Next we build a *phenodata* dataframe with experimental design information, 
which are here found in the cytometry data structures.

```{r convertPhenoData}
# update phenoData structure

chLabels <- 
    keyword(Krieg_fs[[1]], "MARKER_INFO")$MARKER_INFO$channel_name
chMarkers <- 
    keyword(Krieg_fs[[1]], "MARKER_INFO")$MARKER_INFO$marker_name
marker_class <- 
    keyword(Krieg_fs[[1]], "MARKER_INFO")$MARKER_INFO$marker_class

chLabels <- chLabels[marker_class != "none"]
chMarkers <- chMarkers[marker_class != "none"]
# marker_class all equal to "type", 24 markers are left

phenoData <- flowCore::pData(Krieg_fs)
additionalPhenoData <- 
    keyword(Krieg_fs[[1]], "EXPERIMENT_INFO")$EXPERIMENT_INFO
phenoData <- cbind(phenoData, additionalPhenoData)
    
flowCore::pData(Krieg_fs) <- phenoData
```

Next, we scale-transform the mass cytometry data and calculate the pairwise 
distance matrix.

```{r distCalc}
# transform flow set (arcsinh(cofactor = 5))
trans <- arcsinhTransform(
    transformationId="ArcsinhTransform", 
    a = 0, 
    b = 1/5, 
    c = 0)

# Applying arcsinh() transformation
Krieg_fs_trans <- transform(
    Krieg_fs,
    transformList(chLabels, trans))

# Calculating Sample distances
pwDist <- pairwiseEMDDist(
    Krieg_fs_trans,
    channels = chMarkers,
    verbose = FALSE
)
```

From the distance matrix, we can now calculate the MDS projection. This is also 
done using the `CytoMDS` package.

```{r MDSCalc}
mdsObj <- computeMetricMDS(
    pwDist,
    seed = 0)

show(mdsObj)
```

Optionally, for the use of biplots as an interpretation tool, sample specific 
statistics can be provided by the user. Here we calculate standard univariate 
statistics from each variable of the multidimensional distribution (again using 
the `CytoMDS` package). 

```{r statsCalc}
# Computing sample statistics

statFUNs <- c(
    "median" = stats::median, 
    "std-dev" = stats::sd,
    "mean" = base::mean,
    "Q25" = function(x, na.rm) 
        stats::quantile(x, probs = 0.25, na.rm = na.rm),
    "Q75" = function(x, na.rm) 
        stats::quantile(x, probs = 0.75, na.rm = na.rm))

chStats <- CytoMDS::channelSummaryStats(
    Krieg_fs_trans,
    channels = chMarkers,
    statFUNs = statFUNs)

```

The newly created objects can be saved as .rds files.  
These files can in turn be selected within the shiny app for visualization.  

```{r saveRDS, eval=FALSE}
saveRDS(mdsObj, file = "Krieg_mdsObj.rds")
saveRDS(phenoData, file = "Krieg_phenoData.rds")
saveRDS(chStats, file = "Krieg_chStats.rds")
```

# Visualization of the MDS projection

The `MDSvis` function `run_app` launches the interactive Shiny app and the
three tabs window in the figure below gets opened. 

```{r launch, eval = FALSE}
MDSvis::mdsvis_app()

# alternatively launch the application in demo mode, with the Krieg_Anti_PD_1
# dataset already loaded.
#MDSvis::mdsvis_app(preLoadDemoDataset = TRUE)

```

In the 'Input files' tab the objects can be loaded for visualization. The plots
are then shown in the 'View' tab while the 'General settings' contains more
technical plot settings controls. All the input files are expected to
have .rds extension and at least a file containing the MDS object has
to be loaded. Optionally a phenodata file containing a phenodata dataframe
and/or a file containing a list of statistics for biplot visualization
can be loaded.  

Alternatively, a 'demo mode' (see in commented code chunck above) allows 
the user to launch the app with the *Krieg_Anti_PD_1* dataset pre-loaded, 
without importing input .rds files.

![](images/InputFiles.png)
We can select the previously created files and proceed with the visualization.
Note that when a phenodata file is selected a new control appears allowing
to select only a subset of variables (by default all are selected). These 
phenodata selected variables will be the only ones available in the drop-down 
list controls in the 'View' tab.

![](images/InputFiles2.png)
We can now open the 'View' tab to see the plot of the projection results.
The controls on the side allow to choose the projection axes; colour, label, 
define facet or shape the points according to phenodata variables; add biplot;
show a `plotly` plot for interactive plot exploration or flip axes. 

![](images/View.png)
For example we can colour the points according to group_id, 
label the points according to sample_id, 
and use shapes according to batch_id, as shown in the figure below.

![](images/View2.png)
We can also defne facets according to the two batches, as in the figure below:  

![](images/View3.png)

We can also add a biplot created based on sample statistics by clicking on the
biplot checkbox. The idea is to calculate the correlation of the sample
statistics w.r.t. the axes of projection, so that these correlations can be
represented on a correlation circle overlaid on the projection plot.  

The desired statistic can be selected from the drop down menu and
it is possible to show only the arrows that respect a selected length threshold.  
In the example below, the chosen statistic is the median while the arrow length
threshold is 0.8.

![](images/ViewBiplot.png)

When some data are too large to be displayed as labels, 
one possible solution is to display an interactive `plotly` plot 
below the regular one by selecting the corresponding checkbox.   
We can add new `plotly` tooltips and highlight the corresponding information 
for each point by hovering over them.

![](images/ViewPlotly.png)

Finally, the obtained plots can be exported as a pdf file. 
The user can defined the corresponding plot title, pdf file name 
and plot size (with and height) to be used. 

![](images/PDFExport.png)

# General settings

For completeness we show below the 'General settings' tab which
contains some general controls regarding e.g. points features, the
corresponding labels and biplot arrows. For more details see the
`CytoMDS::ggplotSampleMDS()` function parameters.

![](images/GeneralSettings.png)

# Session information {-}

```{r sessioninfo, echo=FALSE}
sessionInfo()
```

# References {-}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mdsvis_app.R
\name{mdsvis_app}
\alias{mdsvis_app}
\title{Launch shiny app for MDS projection visualization}
\usage{
mdsvis_app(preLoadDemoDataset = FALSE)
}
\arguments{
\item{preLoadDemoDataset}{if TRUE, pre-load the \emph{Krieg_Anti_PD_1} dataset}
}
\value{
no return value
}
\description{
Launch shiny app for MDS projection visualization
}
\examples{
if (interactive()) {
  mdsvis_app()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MDSvis-package.R
\docType{package}
\name{MDSvis}
\alias{MDSvis-package}
\alias{MDSvis}
\title{MDSvis package}
\description{
\code{MDSvis} provides a shiny application for interactive visualisation of
Multi Dimensional Scaling (MDS) results.
}
\seealso{
\link[CytoMDS:CytoMDS-package]{CytoMDS}

\link{mdsvis_app}
}
\author{
\strong{Maintainer}: Philippe Hauchamps \email{philippe.hauchamps@uclouvain.be} (\href{https://orcid.org/0000-0003-2865-1852}{ORCID})

Authors:
\itemize{
  \item Andrea Vicini \email{andrea.vicini@uclouvain.be} (\href{https://orcid.org/0000-0001-9438-6909}{ORCID})
  \item Laurent Gatto \email{laurent.gatto@uclouvain.be} (\href{https://orcid.org/0000-0002-1520-2268}{ORCID})
}

Other contributors:
\itemize{
  \item Shabnam Zaman \email{shabnam.zaman@uclouvain.be} (\href{https://orcid.org/0000-0001-5399-0130}{ORCID}) [contributor]
}

}
\keyword{internal}
